@@ -36,6 +36,7 @@ try:
     from cStringIO import StringIO
 except ImportError:
     from io import StringIO
+from io import BytesIO
 import cv2
 import cv_bridge
 import image_geometry
@@ -153,6 +154,21 @@ def _get_corners(img, board, refine = True, checkerboard_flags=0):
     if not all([(BORDER < corners[i, 0, 0] < (w - BORDER)) and (BORDER < corners[i, 0, 1] < (h - BORDER)) for i in range(corners.shape[0])]):
         ok = False
 
+    # Ensure that all corner-arrays are going from top to bottom.
+    if board.n_rows!=board.n_cols:
+        if corners[0, 0, 1] > corners[-1, 0, 1]:
+            corners = numpy.copy(numpy.flipud(corners))
+    else:
+        direction_corners=(corners[-1]-corners[0])>=numpy.array([[0.0,0.0]])
+    
+        if not numpy.all(direction_corners):
+            if not numpy.any(direction_corners):
+                corners = numpy.copy(numpy.flipud(corners))
+            elif direction_corners[0][0]:
+                corners=numpy.rot90(corners.reshape(board.n_rows,board.n_cols,2)).reshape(board.n_cols*board.n_rows,1,2)
+            else:
+                corners=numpy.rot90(corners.reshape(board.n_rows,board.n_cols,2),3).reshape(board.n_cols*board.n_rows,1,2)
+
     if refine and ok:
         # Use a radius of half the minimum distance between corners. This should be large enough to snap to the
         # correct corner, but not so large as to include a wrong corner in the search window.
@@ -202,7 +218,8 @@ class Calibrator(object):
     """
     Base class for calibration system
     """
-    def __init__(self, boards, flags=0, pattern=Patterns.Chessboard, name='', checkerboard_flags=cv2.CALIB_CB_FAST_CHECK):
+    def __init__(self, boards, flags=0, pattern=Patterns.Chessboard, name='', 
+    checkerboard_flags=cv2.CALIB_CB_FAST_CHECK, max_chessboard_speed = -1.0):
         # Ordering the dimensions for the different detectors is actually a minefield...
         if pattern == Patterns.Chessboard:
             # Make sure n_cols > n_rows to agree with OpenCV CB detector output
@@ -231,6 +248,8 @@ class Calibrator(object):
         self.goodenough = False
         self.param_ranges = [0.7, 0.7, 0.4, 0.5]
         self.name = name
+        self.last_frame_corners = None
+        self.max_chessboard_speed = max_chessboard_speed
 
     def mkgray(self, msg):
         """
@@ -240,7 +259,7 @@ class Calibrator(object):
         # TODO: get a Python API in cv_bridge to check for the image depth.
         if self.br.encoding_to_dtype_with_channels(msg.encoding)[0] in ['uint16', 'int16']:
             mono16 = self.br.imgmsg_to_cv2(msg, '16UC1')
-            mono8 = numpy.array(numpy.clip(mono16, 0, 255), dtype=numpy.uint8)
+            mono8 = numpy.array(mono16 / 256, dtype=numpy.uint8)
             return mono8
         elif 'FC1' in msg.encoding:
             # floating point image handling
@@ -248,9 +267,9 @@ class Calibrator(object):
             _, max_val, _, _ = cv2.minMaxLoc(img)
             if max_val > 0:
                 scale = 255.0 / max_val
-                mono_img = (img * scale).astype(np.uint8)
+                mono_img = (img * scale).astype(numpy.uint8)
             else:
-                mono_img = img.astype(np.uint8)
+                mono_img = img.astype(numpy.uint8)
             return mono_img
         else:
             return self.br.imgmsg_to_cv2(msg, "mono8")
@@ -273,7 +292,21 @@ class Calibrator(object):
         params = [p_x, p_y, p_size, skew]
         return params
 
-    def is_good_sample(self, params):
+    def is_slow_moving(self, corners, last_frame_corners):
+        """
+        Returns true if the motion of the checkerboard is sufficiently low between
+        this and the previous frame.
+        """
+        # If we don't have previous frame corners, we can't accept the sample
+        if last_frame_corners is None:
+            return False
+        num_corners = len(corners)
+        corner_deltas = (corners - last_frame_corners).reshape(num_corners, 2)
+        # Average distance travelled overall for all corners
+        average_motion = numpy.average(numpy.linalg.norm(corner_deltas, axis = 1))
+        return average_motion <= self.max_chessboard_speed
+
+    def is_good_sample(self, params, corners, last_frame_corners):
         """
         Returns true if the checkerboard detection described by params should be added to the database.
         """
@@ -287,7 +320,15 @@ class Calibrator(object):
         d = min([param_distance(params, p) for p in db_params])
         #print "d = %.3f" % d #DEBUG
         # TODO What's a good threshold here? Should it be configurable?
-        return d > 0.2
+        if d <= 0.2:
+            return False
+
+        if self.max_chessboard_speed > 0:
+            if not self.is_slow_moving(corners, last_frame_corners):
+                return False
+
+        # All tests passed, image should be good for calibration
+        return True
 
     _param_names = ["X", "Y", "Size", "Skew"]
 
@@ -506,7 +547,7 @@ def image_from_archive(archive, name):
     Used for tarfile loading and unit test.
     """
     member = archive.getmember(name)
-    imagefiledata = numpy.fromstring(archive.extractfile(member).read(-1), numpy.uint8)
+    imagefiledata = numpy.fromstring(archive.extractfile(member).read(), numpy.uint8)
     imagefiledata.resize((1, imagefiledata.size))
     return cv2.imdecode(imagefiledata, cv2.IMREAD_COLOR)
 
@@ -751,11 +792,12 @@ class MonoCalibrator(Calibrator):
 
                 # Add sample to database only if it's sufficiently different from any previous sample.
                 params = self.get_parameters(corners, board, (gray.shape[1], gray.shape[0]))
-                if self.is_good_sample(params):
+                if self.is_good_sample(params, corners, self.last_frame_corners):
                     self.db.append((params, gray))
                     self.good_corners.append((corners, board))
                     print(("*** Added sample %d, p_x = %.3f, p_y = %.3f, p_size = %.3f, skew = %.3f" % tuple([len(self.db)] + params)))
-
+        
+        self.last_frame_corners = corners
         rv = MonoDrawable()
         rv.scrib = scrib
         rv.params = self.compute_goodenough()
@@ -782,7 +824,10 @@ class MonoCalibrator(Calibrator):
         """ Write images and calibration solution to a tarfile object """
 
         def taradd(name, buf):
-            s = StringIO(buf)
+            if isinstance(buf, basestring):
+                s = StringIO(buf)
+            else:
+                s = BytesIO(buf)
             ti = tarfile.TarInfo(name)
             ti.size = len(s.getvalue())
             ti.uname = 'calibrator'
@@ -1052,7 +1097,7 @@ class StereoCalibrator(Calibrator):
                 cv2.drawChessboardCorners(rscrib, (rboard.n_cols, rboard.n_rows), scrib_src, True)
 
             # Report epipolar error
-            if lcorners is not None and rcorners is not None:
+            if lcorners is not None and rcorners is not None and len(lcorners) == len(rcorners):
                 epierror = self.epipolar_error(lundistorted, rundistorted)
 
         else:
@@ -1067,13 +1112,14 @@ class StereoCalibrator(Calibrator):
                                          rdownsampled_corners, True)
 
             # Add sample to database only if it's sufficiently different from any previous sample
-            if lcorners is not None and rcorners is not None:
+            if lcorners is not None and rcorners is not None and len(lcorners) == len(rcorners):
                 params = self.get_parameters(lcorners, lboard, (lgray.shape[1], lgray.shape[0]))
-                if self.is_good_sample(params):
+                if self.is_good_sample(params, lcorners, self.last_frame_corners):
                     self.db.append( (params, lgray, rgray) )
                     self.good_corners.append( (lcorners, rcorners, lboard) )
                     print(("*** Added sample %d, p_x = %.3f, p_y = %.3f, p_size = %.3f, skew = %.3f" % tuple([len(self.db)] + params)))
-
+        
+        self.last_frame_corners = lcorners
         rv = StereoDrawable()
         rv.lscrib = lscrib
         rv.rscrib = rscrib
@@ -1102,7 +1148,10 @@ class StereoCalibrator(Calibrator):
                [("right-%04d.png" % i, im) for i,(_, _, im) in enumerate(self.db)])
 
         def taradd(name, buf):
-            s = StringIO(buf)
+            if isinstance(buf, basestring):
+                s = StringIO(buf)
+            else:
+                s = BytesIO(buf)
             ti = tarfile.TarInfo(name)
             ti.size = len(s.getvalue())
             ti.uname = 'calibrator'
